﻿using Opc.Ua;
using OpcLabs.BaseLib.IdentityModel.User;
using OpcLabs.BaseLib.Instrumentation;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.Application;
using OpcLabs.EasyOpc.UA.Engine;
using OpcLabs.EasyOpc.UA.Extensions;
using OpcLabs.EasyOpc.UA.OperationModel;
using System.ServiceModel;

class Program
{
   static void Main()
   {
      UAEndpointDescriptor endpointDescriptor = "opc.tcp://thederw10.bauercontrols.local:49321";
      endpointDescriptor.EndpointSelectionPolicy = new UAEndpointSelectionPolicy();
      var client = new EasyUAClient();


      // Hook static events
      EasyUAClient.LogEntry += EasyUAClientOnLogEntry;
      try
      {
         // Set the application name, which determines the subject of the client certificate.
         // Note that this only works once in each host process
         // Do something - invoke an OPC read, to trigger some loggable entries.

         try
         {
            EasyUAClient.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AllowUserAcceptCertificate = true;
            EasyUAClient.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AcceptAnyCertificate = false;
            EasyUAApplication.Instance.ApplicationParameters.ApplicationManifest.InstanceOwnStorePath = "C:/TestingMyOwnCertStore";

            X509CertificateTokenInfo cert = new X509CertificateTokenInfo();

            //endpointDescriptor.UserIdentity.UserNameTokenInfo.UserName = "BauerClient";
            //endpointDescriptor.UserIdentity.UserNameTokenInfo.Password = "admin@bauercontrols";
            var attribute = client.Read(endpointDescriptor.WithX509CertificateIdentity("C:\\TestingMyOwnCertStore\\certs\\f690068f8490ec7af468c83373149ff45147b5d3.der", string.Empty), "nsu=KEPServerEX ;ns=2;s=Channel1.Device1.Tag1");

            Console.WriteLine($"Tag Value :{0}", attribute.Value);
         }
         catch (UAException uaException)
         {
            Console.WriteLine("*** Failure: {0}", uaException.GetBaseException().Message);
         }

         // The certificate will be located or created in a directory similar to:
         // C:\ProgramData\OPC Foundation\CertificateStores\MachineDefault\certs
         // and its subject will be as given by the application name.


         Console.WriteLine("Processing log entry events for 10 seconds...");
         System.Threading.Thread.Sleep(10 * 1000);

         Console.WriteLine("Finished.");
      }
      finally
      {
         // Unhook static events
         EasyUAClient.LogEntry -= EasyUAClientOnLogEntry;
      }
   }


   private static void EasyUAClientOnLogEntry(object sender, LogEntryEventArgs logEntryEventArgs)
   {
      if (logEntryEventArgs.EventId == 161)
         Console.WriteLine(logEntryEventArgs);
   }
   private static void ReadValue(string identifier, int arrayLength, int startingIndex, UAEndpointDescriptor endpointDescriptor, EasyUAClient client)
   {
      if (string.IsNullOrWhiteSpace(identifier))
         throw new ArgumentNullException(nameof(identifier));

      object rv = null;
      try
      {
         var readResponse = client.Read(new UAReadArguments(endpointDescriptor, identifier, UAIndexRangeList.OneDimension(startingIndex, startingIndex + arrayLength - 1)));
         var response = (int)readResponse.StatusCode.ToInt64();
         rv = readResponse.Value;

         Console.WriteLine($"Tag Information - Response: {response} Response Value: {rv}");
      }
      catch (Exception exception)
      {
         string message = $"OPC-UA Read value operation has failed (Node Id - {identifier}).";
         rv = null;
      }
   }


}
